#!usr/bin/env python
# -*- coding:utf-8 -*-
"""
qcc 请求头生成
"""
import hashlib
import hmac
import json
from urllib import parse
from urllib.parse import urlparse, parse_qs, urlencode


class SignTool(object):
    def __init__(self):
        self.seeds = {
            "0": "W", "1": "l", "2": "k", "3": "B", "4": "Q", "5": "g", "6": "f", "7": "i", "8": "i", "9": "r",
            "10": "v", "11": "6", "12": "A", "13": "K", "14": "N", "15": "k", "16": "4", "17": "L", "18": "1", "19": "8"
        }
        self.n = 20
        self.tid = ""

    def generate_map_result(self, s):
        if not s:
            s = "/"
        s = s.lower()
        s = s + s
        k = ''
        for i in s:
            k += self.seeds[str(ord(i) % 20)]
        return k

    @staticmethod
    def sign_with_hmac(key, s):
        # 使用HmacSHA512算法
        return hmac.new(bytes(key, encoding='utf-8'), bytes(s, encoding='utf-8'), hashlib.sha512).hexdigest()

    def get_head_key(self, s, query_params=None, data=None):
        s = s.lower()
        map_result = self.generate_map_result(s)
        if query_params:
            query_string = parse.urlencode(query_params)
            s += "?" + query_string
        if not data:
            data_s = "{}"
        else:
            query_params_lower = {k.lower(): v.lower() for k, v in data.items()}
            data_s = json.dumps(query_params_lower, separators=(',', ':'), ensure_ascii=False).lower()
        hmac_result = self.sign_with_hmac(map_result, s + data_s).lower()
        key = hmac_result[8:28]
        return key

    def get_head_value(self, url, t="", data=None):
        if not url:
            url = "/"
        if not data:
            data = {}

        parsed_url = urlparse(url)
        key = parsed_url.path.lower()

        query_params = parse_qs(parsed_url.query)

        if query_params:  # GET request
            query_params_lower = {k.lower(): v[0].lower() for k, v in query_params.items()}
            data_s = "{}"
            query_string = urlencode(query_params_lower)
            key += "?" + query_string
        else:  # POST request
            query_params_lower = {k.lower(): v.lower() for k, v in data.items()}
            data_s = json.dumps(query_params_lower, separators=(',', ':'), ensure_ascii=False).lower()

        enc_data = f"{key}pathString{data_s}{t}"
        enc_key = self.generate_map_result(key)
        result = self.sign_with_hmac(enc_key, enc_data)
        return result

    def get_header(self, url, data=None):
        paths = parse.urlparse(url)
        if not paths.query:
            uri = paths.path
            query_data = {}
        else:
            uri = paths.path + "?" + paths.query
            query_data = dict(parse.parse_qsl(paths.query))

        header_key = self.get_head_key(uri, data=data)
        t = self.tid
        header_val = self.get_head_value(uri, t, query_data)
        return {header_key: header_val}

if __name__ == '__main__':
    sign_tool = SignTool()
    # sign_tool.tid = get_tid()
    sign_tool.tid = "8a2ac1194d00dbbd156b2a0cd7fdd36f"
    print(sign_tool.get_header('https://www.qcc.com/api/goods/getSVIPGoodsList'))
